/*
 *      listdialog.h
 *
 *      Copyright 2015 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef LISTDIALOG_H
#define	LISTDIALOG_H

#include "defs.h"
#include "fxext.h"

class ChannelsList : public FXIconList
{
    FXDECLARE(ChannelsList)
protected:
    ChannelsList();
private:
    ChannelsList(const ChannelsList&);
    ChannelsList& operator=(const ChannelsList&);
public:
    ChannelsList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=ICONLIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
    enum {
        ID_SORT_BY_CHANNEL = FXIconList::ID_LAST,
        ID_SORT_BY_MEMBERS,
        ID_SORT_BY_TOPIC
    };
    
    long onCmdHeader(FXObject*,FXSelector,void*);
    long onUpdHeader(FXObject*,FXSelector,void*);
    long onCmdSortByChannel(FXObject*,FXSelector,void*);
    long onCmdSortByMembers(FXObject*,FXSelector,void*);
    long onCmdSortByTopic(FXObject*,FXSelector,void*);
    static FXint ascendingMembers(const FXIconItem* a, const FXIconItem* b);
    static FXint descendingMembers(const FXIconItem* a, const FXIconItem* b);
    static FXint ascendingTopic(const FXIconItem* a, const FXIconItem* b);
    static FXint descendingTopic(const FXIconItem* a, const FXIconItem* b);
};

class ListDialog: public FXDialogBox
{
    FXDECLARE(ListDialog)
public:
    ListDialog(FXMainWindow *owner, IrcEngine *engine);
    virtual ~ListDialog();

    virtual void create();

    long onJoin(FXObject*,FXSelector,void*);
    long onCancel(FXObject*,FXSelector,void*);
    long onClose(FXObject*,FXSelector,void*);
    long onTableDoubleClick(FXObject*,FXSelector,void*);
    long onTableHeader(FXObject*,FXSelector,void*);
    long onKeyPress(FXObject*,FXSelector,void*);
    long onSearch(FXObject*,FXSelector,void*);
private:
    ListDialog():m_engine(NULL),m_contents(NULL),m_tableframe(NULL),m_buttonframe(NULL),m_table(NULL)
    ,m_buttonJoin(NULL),m_buttonClose(NULL)
    {}
    ListDialog(const ListDialog& orig);

    void fillChannelList();

    IrcEngine *m_engine;
    FXVerticalFrame *m_contents, *m_tableframe;
    FXHorizontalFrame *m_buttonframe, *m_searchframe;
    ChannelsList *m_table;
    dxEXButton *m_buttonJoin, *m_buttonClose;
    dxStringArray m_channels;
    FXTextField *m_search;
};

#endif	/* LISTDIALOG_H */

