/*
 *      defs.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DEFS_H
#define DEFS_H

#include <QString>
#include <QColor>

#define LUA_HELP_PATH "http://www.dxirc.org/dxirc-lua.html"
#define TAB (QString)"\t"

struct lua_State;

enum TYPE {
    SERVER,
    CHANNEL,
    QUERY,
    OTHER,
    DCC,
    BOATS
};

enum DCCTYPE {
    DCC_NONE,
    DCC_CHATIN, //connect to someone for chat
    DCC_CHATOUT, //someone connect to me for chat
    DCC_IN, //connect to someone for file receive
    DCC_OUT, //someone connect to me for file send
    DCC_PIN, //someone connect to me for file receive (PSEND)
    DCC_POUT //connect to someone for file send (PSEND)
};

enum LuaCommands {
    LUA_LOAD,
    LUA_UNLOAD,
    LUA_COMMAND,
    LUA_LIST
};

enum UserMode {
    OWNER = 0,
    ADMIN,
    OP,
    VOICE,
    HALFOP,
    NONE
};

struct NickInfo {
    QString nick;
    QString user;
    QString real;
    QString host;
    bool away;
};

//used for ignoreuser, tracking friends
struct UserInfo {
    QString nick;
    QString channel;
    QString network;
};

struct ServerInfo {
    QString name;
    QString hostname;
    int port;
    QString nick;
    QString realname;
    QString passwd;
    QString channels;
    QString commands;
    bool autoConnect;
    bool useSsl;
};

struct IrcColor {
    QColor text;
    QColor back;
    QColor user;
    QColor action;
    QColor notice;
    QColor error;
    QColor hilight;
    QColor link;
    QColor mymsg;
    QColor log;
};

struct LuaRequest {
    LuaCommands type;
    QString text;
};

struct LuaScript {
    QString path;
    QString name;
    QString version;
    QString description;
    lua_State *L;
};

struct LuaScriptCommand {
    QString name; //command name
    QString funcname;
    QString helptext;
    QString script; //script name
};

struct LuaScriptEvent {
    QString name; //eventname, e.g. PRIVMSG, JOIN or ALL etc.
    QString funcname;
    QString script; //script name
};

struct DccFile {
    QString path;
    DCCTYPE type;
    quint32 currentPosition;
    quint32 speed;
    quint32 size;
    bool started;
    bool canceled;
    bool finished;
    quint32 finishedPosition; //used in sending for handle user position
    int token; //token passive send
    QString ip; //ip address
    int port;
    QString nick; //usefull for resume
    bool operator==(const DccFile& file) const
    {
        return path==file.path && type==file.type && ip==file.ip && port==file.port && token==file.token && nick==file.nick;
    }
};

#endif /* DEFS_H_ */
