/*
 *      serverdialog.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef SERVERDIALOG_H
#define SERVERDIALOG_H

#include <QtGui>
#if(QT_VERSION > 0x050000)
#include <QtWidgets>
#endif
#include "defs.h"

class EditDialog : public QDialog
{
    Q_OBJECT
public:
    EditDialog(ServerInfo info, QString title, QWidget *parent = 0);
    ServerInfo getInfo() const;
private:
    QLineEdit           *m_network;
    QLineEdit           *m_hostname;
    QSpinBox            *m_port;
    QLineEdit           *m_password;
    QLineEdit           *m_nick;
    QLineEdit           *m_realname;
    QLineEdit           *m_channels;
    QTextEdit           *m_commands;
    QCheckBox           *m_ssl;
    QCheckBox           *m_autoconnect;
};

class ServerDialog : public QDialog
{
    Q_OBJECT
public:
    explicit ServerDialog(QWidget *parent = 0);
signals:
    void connectServer(ServerInfo);
private slots:
    void slotChangeServer();
    void slotJoin();
    void slotAdd();
    void slotModify();
    void slotDelete();
    void slotSave();
private:
    QListWidget *m_serverList;
    QLabel *m_hostname;
    QLabel *m_port;
    QLabel *m_nick;
    QLabel *m_realname;
    QLabel *m_password;
    QLabel *m_channels;
    QLabel *m_commands;
    QLabel *m_ssl;
    QLabel *m_autoconnect;
    QPushButton *m_buttonJoin, *m_buttonAdd, *m_buttonModify, *m_buttonDelete, *m_buttonSaveClose;
    QList<ServerInfo> m_servers;
    void updateList();
    void updateDetails();
    bool hostnameExist(const QString &hostname, const int &port, const QString &nick);
};

#endif // SERVERDIALOG_H
